import pygame
import random
import sys

# --- 1. 初始化 Pygame ---
pygame.init()

# --- 2. 定义常量 ---
SCREEN_WIDTH = 800
SCREEN_HEIGHT = 600
GRID_SIZE = 20 # 每个方块的大小
GRID_WIDTH = SCREEN_WIDTH // GRID_SIZE
GRID_HEIGHT = SCREEN_HEIGHT // GRID_SIZE

# 颜色定义 (RGB)
WHITE = (255, 255, 255)
BLACK = (0, 0, 0)
GREEN = (0, 255, 0)
RED = (255, 0, 0)
BLUE = (0, 0, 255)

# 方向定义
UP = (0, -1)
DOWN = (0, 1)
LEFT = (-1, 0)
RIGHT = (1, 0)

# --- 3. 设置窗口 ---
screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))
pygame.display.set_caption("Python 贪吃蛇")

# --- 4. 游戏元素类/函数 ---

class Snake:
    def __init__(self):
        self.length = 1
        self.positions = [((SCREEN_WIDTH // 2), (SCREEN_HEIGHT // 2))] # 蛇的初始位置 (中心)
        self.direction = random.choice([UP, DOWN, LEFT, RIGHT]) # 初始随机方向
        self.color = GREEN

    def get_head_position(self):
        return self.positions[0]

    def turn(self, point):
        # 避免掉头
        if self.length > 1 and \
           (point[0] * -1, point[1] * -1) == self.direction: # 新方向是当前方向的反方向
            return
        else:
            self.direction = point

    def move(self):
        cur = self.get_head_position()
        x, y = self.direction
        new = (((cur[0] + (x * GRID_SIZE)) % SCREEN_WIDTH), (cur[1] + (y * GRID_SIZE)) % SCREEN_HEIGHT) # 移动并处理穿墙

        # 检查是否撞到自己
        if len(self.positions) > 2 and new in self.positions[2:]: # 排除蛇头和紧跟蛇头的第二节
            return True # 返回 True 表示游戏结束
        else:
            self.positions.insert(0, new) # 添加新蛇头
            if len(self.positions) > self.length:
                self.positions.pop() # 移除蛇尾 (如果没吃到食物)
            return False # 游戏继续

    def grow(self):
        self.length += 1

    def draw(self, surface):
        for p in self.positions:
            r = pygame.Rect((p[0], p[1]), (GRID_SIZE, GRID_SIZE))
            pygame.draw.rect(surface, self.color, r)
            pygame.draw.rect(surface, BLACK, r, 1) # 绘制边框

class Food:
    def __init__(self):
        self.position = (0,0)
        self.color = RED
        self.randomize_position()

    def randomize_position(self):
        self.position = (random.randint(0, GRID_WIDTH - 1) * GRID_SIZE,
                         random.randint(0, GRID_HEIGHT - 1) * GRID_SIZE)

    def draw(self, surface):
        r = pygame.Rect((self.position[0], self.position[1]), (GRID_SIZE, GRID_SIZE))
        pygame.draw.rect(surface, self.color, r)
        pygame.draw.rect(surface, BLACK, r, 1)

def draw_grid(surface):
    for y in range(0, SCREEN_HEIGHT, GRID_SIZE):
        pygame.draw.line(surface, BLUE, (0, y), (SCREEN_WIDTH, y))
    for x in range(0, SCREEN_WIDTH, GRID_SIZE):
        pygame.draw.line(surface, BLUE, (x, 0), (x, SCREEN_HEIGHT))

# --- 5. 游戏主循环 ---
def main_game():
    snake = Snake()
    food = Food()
    score = 0
    clock = pygame.time.Clock() # 用于控制帧率

    # 字体设置
    font = pygame.font.Font(None, 36) # 默认字体，大小36

    running = True
    game_over = False

    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT: # 关闭窗口事件
                running = False
            elif event.type == pygame.KEYDOWN: # 按键事件
                if event.key == pygame.K_UP:
                    snake.turn(UP)
                elif event.key == pygame.K_DOWN:
                    snake.turn(DOWN)
                elif event.key == pygame.K_LEFT:
                    snake.turn(LEFT)
                elif event.key == pygame.K_RIGHT:
                    snake.turn(RIGHT)
                elif event.key == pygame.K_ESCAPE: # 按 ESC 键退出
                    running = False

        if not game_over:
            # 移动蛇并检查是否撞到自己
            if snake.move():
                game_over = True
            
            # 检查蛇是否吃到食物
            if snake.get_head_position() == food.position:
                snake.grow()
                score += 10
                food.randomize_position() # 重新生成食物

        # --- 绘制所有元素 ---
        screen.fill(BLACK) # 填充背景
        # draw_grid(screen) # 可以选择绘制网格
        snake.draw(screen)
        food.draw(screen)

        # 显示得分
        score_text = font.render(f"得分: {score}", True, WHITE)
        screen.blit(score_text, (5, 5)) # 显示在左上角

        # 游戏结束时显示信息
        if game_over:
            game_over_text = font.render("游戏结束! 按 ESC 退出", True, RED)
            # 计算文字位置使其居中
            text_rect = game_over_text.get_rect(center=(SCREEN_WIDTH // 2, SCREEN_HEIGHT // 2))
            screen.blit(game_over_text, text_rect)

        pygame.display.flip() # 更新整个屏幕显示

        # 控制游戏帧率
        clock.tick(10) # 10 FPS (每秒移动10次)

    pygame.quit() # 退出 Pygame
    sys.exit() # 退出程序

if __name__ == '__main__':
    main_game()
